import aiosqlite as sl


class database:
    def __init__(self, database_path:str):
        from . import DATABASE_PATH

        if database_path != None:
            self.database = database_path
        else:
            self.database = DATABASE_PATH
        
        if self.database == None:
            raise ValueError("<database_path> cannot be None.")


    async def create_table(self, table_name:str, columns:list):
        if not isinstance(table_name, str):
            raise TypeError("<table_name> needs to be a string.")
            return
        if not isinstance(columns, list):
            raise TypeError("<colums> needs to be a list.")
            return

        cursor = sl.connect(f'{self.database}', isolation_level=None)

        query = f"CREATE TABLE IF NOT EXISTS {table_name} (%s)" % ", ".join(columns)

        async with cursor:
            await cursor.execute('pragma journal_mode=wal')
            await cursor.execute(query)

    
    async def add_column(self, table_name:str, column:str):
        query = f"ALTER TABLE {table_name} ADD COLUMN {column}"
        ...


    async def insert(self, table_name:str, column_value_dict:dict, replace:bool=False):
        if not isinstance(table_name, str):
            raise TypeError("<table_name> needs to be a string.")
            return
        if not isinstance(column_value_dict, dict):
            raise TypeError("<column_value_dict> needs to be a dict.")
            return
        if not isinstance(replace, bool):
            raise TypeError("<replace> needs to be a bool.")
            return

        cursor = sl.connect(f'{self.database}', isolation_level=None)

        columns = tuple(column_value_dict.keys())
        values = tuple(column_value_dict.values())

        if replace == True:
            query = f"INSERT OR REPLACE INTO {table_name} {columns} VALUES {values}"
        else:
            query = f"INSERT INTO {table_name} {columns} VALUES {values}"

        async with cursor:
            await cursor.execute('pragma journal_mode=wal')
            await cursor.execute(query)


    async def select(self, table_name:str, columns:list, condition:str=None):
        if not isinstance(table_name, str):
            raise TypeError("<table_name> needs to be a string.")
            return
        if not isinstance(columns, list):
            raise TypeError("<colums> needs to be a list.")
            return
        if not condition == None:
            if not isinstance(condition, str):
                raise TypeError("<condition> needs to be a string.")
                return

        joined_columns = ",".join(columns)

        if condition == None:
            query = f"SELECT {joined_columns} FROM {table_name}"
        else:
            query = f"SELECT {joined_columns} FROM {table_name} WHERE {condition}"

        cursor = sl.connect(f'{self.database}', isolation_level=None)

        async with cursor:
            await cursor.execute('pragma journal_mode=wal')
            result = await cursor.execute(query)
            rows = await result.fetchall()

        return rows
        

    async def select_join(self, table_name_1:str, table_name_2:str, columns:list, matching_column:str, condition:str=None):
        if not isinstance(table_name_1, str):
            raise TypeError("<table_name_1> needs to be a string.")
            return
        if not isinstance(table_name_2, str):
            raise TypeError("<table_name_2> needs to be a string.")
            return
        if not isinstance(matching_column, str):
            raise TypeError("<matching_column> needs to be a string.")
            return
        if not isinstance(columns, list):
            raise TypeError("<colums> needs to be a list.")
            return
        if not condition == None:
            if not isinstance(condition, str):
                raise TypeError("<condition> needs to be a string.")
                return

        joined_columns = ",".join(columns)

        if condition == None:
            query = f"SELECT {joined_columns} FROM {table_name_1} INNER JOIN {table_name_2} on {table_name_2}.{matching_column} = {table_name_1}.{matching_column}"
        else:
            query = f"SELECT {joined_columns} FROM {table_name_1} WHERE {condition}"

        cursor = sl.connect(f'{self.database}', isolation_level=None)

        async with cursor:
            await cursor.execute('pragma journal_mode=wal')
            result = await cursor.execute(query)
            rows = await result.fetchall()

        return rows


    async def query(self, query:str):
        if not isinstance(query, str):
            raise TypeError("<query> needs to be a string.")
            return

        cursor = sl.connect(f'{self.database}', isolation_level=None)

        async with cursor:
            await cursor.execute('pragma journal_mode=wal')
            await cursor.execute(query)
